searchNodes=[{"doc":"Convenience macros to work with Aja's data structures.Use import A to import everything, or import only the macros you need.","ref":"A.html","title":"A","type":"module"},{"doc":"Convenience macro to work with A.ExRanges (exclusive ranges).Use import A to use it, or import A, only: [{:~&gt;, 2}].Examplesiex&gt; 1 ~&gt; 5 1 ~&gt; 5 iex&gt; start ~&gt; stop = 0 ~&gt; 10 iex&gt; {start, stop} {0, 10} iex&gt; for i &lt;- 0 ~&gt; 5, do: &quot;id_\#{i}&quot; [&quot;id_0&quot;, &quot;id_1&quot;, &quot;id_2&quot;, &quot;id_3&quot;, &quot;id_4&quot;]","ref":"A.html#~%3E/2","title":"A.~>/2","type":"macro"},{"doc":"Convenience macro to create or pattern match on A.OrdMaps.Use import A to use it, or import A, only: [{:ord, 1}].Creation examplesiex&gt; ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3}) #A&lt;ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3})&gt; iex&gt; ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;Pattern matching examplesiex&gt; ord(%{b: bat}) = ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt; iex&gt; bat &quot;Bat&quot;Replace existing keys examplesiex&gt; ordered = ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}) iex&gt; ord(%{ordered | b: &quot;Buffalo&quot;}) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; ord(%{ordered | z: &quot;Zebra&quot;}) ** (KeyError) key :z not found in: #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.html#ord/1","title":"A.ord/1","type":"macro"},{"doc":"Exclusive ranges are an exact copy of regular ranges (see Range), except that they exclude the second parameter.Why/when would you need exclusive ranges?The most typical use case would be when using 1..n for loops based on a parameter n &gt;= 0:iex&gt; incorrect = fn n -&gt; for i &lt;- 1..n, do: &quot;id_\#{i}&quot; end iex&gt; incorrect.(3) # works fine in general... [&quot;id_1&quot;, &quot;id_2&quot;, &quot;id_3&quot;] iex&gt; incorrect.(0) # weird bug at edge case! [&quot;id_1&quot;, &quot;id_0&quot;]To fix it, you would typically need to match the n == 0 case and handle it differently, which is adds noise. With exclusive ranges like 0 ~&gt; n, you get to keep the compact and elegant approach from above, while having a correct algorithm that covers the edge case:iex&gt; correct = fn n -&gt; for i &lt;- 1 ~&gt; n + 1, do: &quot;id_\#{i}&quot; end iex&gt; correct.(3) # works fine [&quot;id_1&quot;, &quot;id_2&quot;, &quot;id_3&quot;] iex&gt; correct.(0) # edge case works fine too []Exclusive ranges can be either increasing (start &lt;= stop) or decreasing (start &gt; stop). The start parameter is included (except if start == stop), the stop parameter is always excluded.An exclusive range is represented internally as a struct A.ExRange{start: start, stop: stop} and can be used as is.The A.~&gt;/2 convenience macro makes it possible to have a more compact syntax, similar to ../2. It is totally optional and needs to be imported:iex&gt; import A iex&gt; import A, only: [{:~&gt;, 2}] # more selectiveExamples:iex&gt; A.ExRange.new(5) #A&lt;0 ~&gt; 5&gt; iex&gt; range = 0 ~&gt; 5 #A&lt;0 ~&gt; 5&gt; iex&gt; start ~&gt; stop = range iex&gt; {start, stop} iex&gt; {0, 4} iex&gt; Enum.to_list(range) [0, 1, 2, 3, 4] iex&gt; Enum.count(range) 5 iex&gt; Enum.member?(range, 5) false iex&gt; Enum.member?(range, 4) true iex&gt; Enum.to_list(3 ~&gt; 0) [3, 2, 1]Just like Ranges, such function calls are efficient memory-wise no matter the size of the range. The implementation of the Enumerable protocol uses logic based solely on the endpoints and does not materialize the whole list of integers.","ref":"A.ExRange.html","title":"A.ExRange","type":"module"},{"doc":"Checks if two ranges are disjoint.Examplesiex&gt; A.ExRange.disjoint?(1 ~&gt; 6, 6 ~&gt; 9) true iex&gt; A.ExRange.disjoint?(6 ~&gt; 1, 6 ~&gt; 9) true iex&gt; A.ExRange.disjoint?(1 ~&gt; 6, 5 ~&gt; 9) false iex&gt; A.ExRange.disjoint?(1 ~&gt; 6, 2 ~&gt; 7) false","ref":"A.ExRange.html#disjoint?/2","title":"A.ExRange.disjoint?/2","type":"function"},{"doc":"Creates a new exclusive range. start defaults to 0.Examplesiex&gt; A.ExRange.new(0, 100) #A&lt;0 ~&gt; 100&gt; iex&gt; A.ExRange.new(10) #A&lt;0 ~&gt; 10&gt;","ref":"A.ExRange.html#new/2","title":"A.ExRange.new/2","type":"function"},{"doc":"","ref":"A.ExRange.html#t:t/0","title":"A.ExRange.t/0","type":"type"},{"doc":"Some extra helper functions for working with IO data, that are not in the core IO module.","ref":"A.IO.html","title":"A.IO","type":"module"},{"doc":"Checks if IO data is empty in &quot;constant&quot; time.Should only need to loop until it finds one character or binary to stop, unlike IO.iodata_length(iodata) == 0 which needs to perform the complete loop to compute the length first.Examplesiex&gt; A.IO.iodata_empty?([&quot;&quot;, []]) true iex&gt; A.IO.iodata_empty?(&#39;a&#39;) false iex&gt; A.IO.iodata_empty?([&quot;a&quot;]) false iex&gt; A.IO.iodata_empty?([&quot;&quot;, [], [&quot;&quot; | &quot;c&quot;]]) falseRationaleEven if IO.iodata_length/1 is a very efficient BIF implemented in C, it has a linear algorithmic complexity and can become slow if invoked on an IO list with many elements.This is not a far-fetched scenario, and a production use case can easily include &quot;big&quot; IO-lists with:JSON encoding to IO-data of long lists / nested objectsloops within HTML templates","ref":"A.IO.html#iodata_empty?/1","title":"A.IO.iodata_empty?/1","type":"function"},{"doc":"Some extra helper functions for working with integers, that are not in the core Integer module.","ref":"A.Integer.html","title":"A.Integer","type":"module"},{"doc":"Format integers for humans, with thousand separators.Examplesiex&gt; A.Integer.decimal_format(1_234_567) &quot;1,234,567&quot; iex&gt; A.Integer.decimal_format(-123) &quot;-123&quot; iex&gt; A.Integer.decimal_format(-1_234, separator: ?_) &quot;-1_234&quot;","ref":"A.Integer.html#decimal_format/2","title":"A.Integer.decimal_format/2","type":"function"},{"doc":"Returns both the result of div/2 and rem/2 at once.Examplesiex&gt; A.Integer.div_rem(7, 3) {2, 1} iex&gt; A.Integer.div_rem(-99, 2) {-49, -1} iex&gt; A.Integer.div_rem(100, 0) ** (ArithmeticError) bad argument in arithmetic expression","ref":"A.Integer.html#div_rem/2","title":"A.Integer.div_rem/2","type":"function"},{"doc":"Some extra helper functions for working with lists, that are not in the core List module.","ref":"A.List.html","title":"A.List","type":"module"},{"doc":"Populates a list of size n by calling generator_fun repeatedly.Examples# Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand.seed(:exsplus, {1, 2, 3}) iex&gt; A.List.repeatedly(&amp;:rand.uniform/0, 3) [0.40502929729990744, 0.45336720247823126, 0.04094511692041057] # It is basically just syntactic sugar for the following: iex&gt; Stream.repeatedly(&amp;:rand.uniform/0) |&gt; Enum.take(3)RationaleIt has a consistent API with Stream.repeatedly/1 and List.duplicate/2It provides a less verbose way of writing one of the most common uses of Stream.repeatedly/1It removes the temptation to write the following, which is more concise but is technically incorrect: iex&gt; incorrect = fn n -&gt; for _i &lt;- 1..n, do: :rand.uniform() end iex&gt; incorrect.(0) |&gt; length() 2 # because: iex&gt; Enum.to_list(1..0) [1, 0]This is the same problem that A.ExRange is addressing.","ref":"A.List.html#repeatedly/2","title":"A.List.repeatedly/2","type":"function"},{"doc":"A Map preserving key insertion order, with efficient lookups and updates.Works just like regular maps, except that the insertion order is preserved:iex&gt; %{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3} %{&quot;one&quot; =&gt; 1, &quot;three&quot; =&gt; 3, &quot;two&quot; =&gt; 2} iex&gt; A.OrdMap.new([{&quot;one&quot;, 1}, {&quot;two&quot;, 2}, {&quot;three&quot;, 3}]) #A&lt;ord(%{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3})&gt;There is an unavoidable overhead compared to natively implemented maps, so keep using regular maps when you do not care about the insertion order.A.OrdMap:provides efficient (logarithmic) access: it is not a simple list of tuplesimplements the Access behaviour, Enum / Inspect / Collectable protocolsoptionally implements the Jason.Encoder protocol if Jason is installedExamplesA.OrdMap offers the same API as Map :iex&gt; ord_map = A.OrdMap.new([b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;]) #A&lt;ord(%{b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.get(ord_map, :c) &quot;Cat&quot; iex&gt; A.OrdMap.fetch(ord_map, :a) {:ok, &quot;Ant&quot;} iex&gt; A.OrdMap.put(ord_map, :d, &quot;Dinosaur&quot;) #A&lt;ord(%{b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;})&gt; iex&gt; A.OrdMap.put(ord_map, :b, &quot;Buffalo&quot;) #A&lt;ord(%{b: &quot;Buffalo&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.delete(ord_map, :b) #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; Enum.to_list(ord_map) [b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;] iex&gt; [d: &quot;Dinosaur&quot;, b: &quot;Buffalo&quot;, e: &quot;Eel&quot;] |&gt; Enum.into(ord_map) #A&lt;ord(%{b: &quot;Buffalo&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;, e: &quot;Eel&quot;})&gt;Tree-specific functionsDue to its sorted nature, A.OrdMap also offers some extra methods not present in Map, like:first/1 and last/1 to efficiently retrieve the first / last key-value pairpop_first/1 and pop_last/1 to efficiently pop the first / last key-value pairfoldl/3 and foldr/3 to efficiently fold (reduce) from left-to-right or right-to-leftExamples:iex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.first(ord_map) {:b, &quot;Bat&quot;} iex&gt; {:c, &quot;Cat&quot;, updated} = A.OrdMap.pop_last(ord_map) iex&gt; updated #A&lt;ord(%{b: &quot;Bat&quot;, a: &quot;Ant&quot;})&gt; iex&gt; A.OrdMap.foldr(ord_map, [], fn {_key, value}, acc -&gt; [value &lt;&gt; &quot;man&quot; | acc] end) [&quot;Batman&quot;, &quot;Antman&quot;, &quot;Catman&quot;]Access behaviourA.OrdMap implements the Access behaviour.iex&gt; ord_map = A.OrdMap.new([a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;]) iex&gt; ord_map[:a] &quot;Ant&quot; iex&gt; put_in(ord_map[:b], &quot;Buffalo&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; put_in(ord_map[:d], &quot;Dinosaur&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;})&gt; iex&gt; {&quot;Cat&quot;, updated} = pop_in(ord_map[:c]) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;})&gt;Convenience ord/1 macroThe A.OrdMap module can be used without any macro. The A.ord/1 macro does however provide some syntactic sugar to make it more convenient to work with ordered maps, namely:construct new ordered maps without the clutter of a tuple listpattern match on key-values like regular mapsupdate some existing keysExamples:iex&gt; import A iex&gt; ord_map = ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3}) #A&lt;ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3})&gt; iex&gt; ord(%{&quot;三&quot; =&gt; three, &quot;一&quot; =&gt; one}) = ord_map iex&gt; {one, three} {1, 3} iex&gt; ord(%{ord_map | &quot;二&quot; =&gt; &quot;NI!&quot;}) #A&lt;ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; &quot;NI!&quot;, &quot;三&quot; =&gt; 3})&gt;Note: pattern-matching on keys doesn't care about the insertion order.With Jasoniex&gt; A.OrdMap.new([{&quot;un&quot;, 1}, {&quot;deux&quot;, 2}, {&quot;trois&quot;, 3}]) |&gt; Jason.encode!() &quot;{\\&quot;un\\&quot;:1,\\&quot;deux\\&quot;:2,\\&quot;trois\\&quot;:3}&quot;It also preserves the insertion order. Comparing with a regular map:iex&gt; Map.new([{&quot;un&quot;, 1}, {&quot;deux&quot;, 2}, {&quot;trois&quot;, 3}]) |&gt; Jason.encode!() &quot;{\\&quot;deux\\&quot;:2,\\&quot;trois\\&quot;:3,\\&quot;un\\&quot;:1}&quot;There is no way as of now to decode JSON using A.OrdMap.Limitations: equalityA.OrdMap comparisons based on ==/2, ===/2 or the pin operator ^ are UNRELIABLE.In Elixir, pattern-matching and equality for structs work based on their internal representation. While this is a pragmatic design choice that simplifies the language, it means that we cannot rededine how they work for custom data structures.Two ordered maps that are semantically equal (same key-value pairs in the same order) might be considered non-equal when comparing their internals, because there is not a unique way of representing one same map.A.OrdMap.equal?/2 should be used instead:iex&gt; ord_map1 = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;})&gt; iex&gt; ord_map2 = A.OrdMap.new(c: &quot;Cat&quot;, a: &quot;Ant&quot;, b: &quot;Bat&quot;) |&gt; A.OrdMap.delete(:c) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;})&gt; iex&gt; ord_map1 == ord_map2 false iex&gt; A.OrdMap.equal?(ord_map1, ord_map2) true iex&gt; match?(^ord_map1, ord_map2) falsePattern-matching and opaque typeAn A.OrdMap is represented internally using the %A.OrdMap{} struct. This struct can be used whenever there's a need to pattern match on something being an A.OrdMap:iex&gt; match?(%A.OrdMap{}, A.OrdMap.new()) trueNote, however, than A.OrdMap is an opaque type: its struct internal fields must not be accessed directly.As discussed in the previous section, ord/1 makes it possible to pattern match on keys as well as checking the type.Difference with A.RBMapA.OrdMap keeps track of key insertion orderA.RBMap keeps keys sorted in ascending order whatever the insertion order is","ref":"A.OrdMap.html","title":"A.OrdMap","type":"module"},{"doc":"Deletes the entry in ord_map for a specific key.If the key does not exist, returns ord_map unchanged.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.delete(ord_map, :b) #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.delete(ord_map, :z) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#delete/2","title":"A.OrdMap.delete/2","type":"function"},{"doc":"Drops the given keys from ord_map.If keys contains keys that are not in ord_map, they're simply ignored.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.drop(ord_map, [:b, :d]) #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#drop/2","title":"A.OrdMap.drop/2","type":"function"},{"doc":"Checks if two ordered maps are equal, meaning they have the same key-value pairs in the same order.Examplesiex&gt; A.OrdMap.equal?(A.OrdMap.new(a: 1, b: 2), A.OrdMap.new(a: 1, b: 2)) true iex&gt; A.OrdMap.equal?(A.OrdMap.new(a: 1, b: 2), A.OrdMap.new(b: 2, a: 1)) false iex&gt; A.OrdMap.equal?(A.OrdMap.new(a: 1, b: 2), A.OrdMap.new(a: 3, b: 2)) false","ref":"A.OrdMap.html#equal?/2","title":"A.OrdMap.equal?/2","type":"function"},{"doc":"Fetches the value for a specific key and returns it in a ok-tuple. If the key does not exist, returns :error.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;) iex&gt; A.OrdMap.fetch(ord_map, :c) {:ok, &quot;C&quot;} iex&gt; A.OrdMap.fetch(ord_map, :z) :error","ref":"A.OrdMap.html#fetch/2","title":"A.OrdMap.fetch/2","type":"function"},{"doc":"Fetches the value for a specific key in the given ord_map, erroring out if ord_map doesn't contain key.If ord_map doesn't contain key, a KeyError exception is raised.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;) iex&gt; A.OrdMap.fetch!(ord_map, :c) &quot;C&quot; iex&gt; A.OrdMap.fetch!(ord_map, :z) ** (KeyError) key :z not found in: #A&lt;ord(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;})&gt;","ref":"A.OrdMap.html#fetch!/2","title":"A.OrdMap.fetch!/2","type":"function"},{"doc":"Finds the fist {key, value} pair in ord_map.Returns a {key, value} tuple if ord_map is non-empty, or nil else.Examplesiex&gt; A.OrdMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.OrdMap.first() {:b, &quot;B&quot;} iex&gt; A.OrdMap.new([]) |&gt; A.OrdMap.first() nil","ref":"A.OrdMap.html#first/1","title":"A.OrdMap.first/1","type":"function"},{"doc":"Folds (reduces) the ordered map from the right with a function. Requires an accumulator.Examplesiex&gt; ord_map = A.OrdMap.new([b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;]) iex&gt; A.OrdMap.foldl(ord_map, &quot;&quot;, fn {_key, value}, acc -&gt; value &lt;&gt; acc end) &quot;AntCatBat&quot; iex&gt; A.OrdMap.foldl(ord_map, [], fn {key, value}, acc -&gt; [{key, value &lt;&gt; &quot;man&quot;} | acc] end) [a: &quot;Antman&quot;, c: &quot;Catman&quot;, b: &quot;Batman&quot;]","ref":"A.OrdMap.html#foldl/3","title":"A.OrdMap.foldl/3","type":"function"},{"doc":"Folds (reduces) the ordered map from the right with a function. Requires an accumulator.Unlike linked lists, this is as efficient as foldl/3. This can typically save a call to Enum.reverse/1 on the result when building a list.Examplesiex&gt; ord_map = A.OrdMap.new([b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;]) iex&gt; A.OrdMap.foldr(ord_map, &quot;&quot;, fn {_key, value}, acc -&gt; value &lt;&gt; acc end) &quot;BatCatAnt&quot; iex&gt; A.OrdMap.foldr(ord_map, [], fn {key, value}, acc -&gt; [{key, value &lt;&gt; &quot;man&quot;} | acc] end) [b: &quot;Batman&quot;, c: &quot;Catman&quot;, a: &quot;Antman&quot;]","ref":"A.OrdMap.html#foldr/3","title":"A.OrdMap.foldr/3","type":"function"},{"doc":"Converts a struct to an ordered map.It accepts the struct module or a struct itself and simply removes the __struct__ field from the given struct or from a new struct generated from the given module.Exampledefmodule User do defstruct [:name, :age] end A.OrdMap.from_struct(User) #A&lt;ord(%{age: nil, name: nil})&gt; A.OrdMap.from_struct(%User{name: &quot;john&quot;, age: 44}) #A&lt;ord(%{age: 44, name: &quot;john&quot;})&gt;","ref":"A.OrdMap.html#from_struct/1","title":"A.OrdMap.from_struct/1","type":"function"},{"doc":"Gets the value for a specific key in ord_map.If key is present in ord_map then its value value is returned. Otherwise, default is returned.If default is not provided, nil is used.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.get(ord_map, :a) &quot;Ant&quot; iex&gt; A.OrdMap.get(ord_map, :z) nil iex&gt; A.OrdMap.get(ord_map, :z, &quot;Zebra&quot;) &quot;Zebra&quot;","ref":"A.OrdMap.html#get/3","title":"A.OrdMap.get/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass.Mirrors Map.get_and_update/3, see its documentation.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;bat&quot;, updated} = A.OrdMap.get_and_update(ord_map, :b, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Buffalo&quot;} ...&gt; end) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; {nil, updated} = A.OrdMap.get_and_update(ord_map, :z, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Zebra&quot;} ...&gt; end) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, z: &quot;Zebra&quot;})&gt; iex&gt; {&quot;Bat&quot;, updated} = A.OrdMap.get_and_update(ord_map, :b, fn _ -&gt; :pop end) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; {nil, updated} = A.OrdMap.get_and_update(ord_map, :z, fn _ -&gt; :pop end) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#get_and_update/3","title":"A.OrdMap.get_and_update/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass.Mirrors Map.get_and_update!/3, see its documentation.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;bat&quot;, updated} = A.OrdMap.get_and_update!(ord_map, :b, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Buffalo&quot;} ...&gt; end) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.get_and_update!(ord_map, :z, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Zebra&quot;} ...&gt; end) ** (KeyError) key :z not found in: #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#get_and_update!/3","title":"A.OrdMap.get_and_update!/3","type":"function"},{"doc":"Gets the value for a specific key in ord_map.If key is present in ord_map then its value value is returned. Otherwise, fun is evaluated and its result is returned.This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; A.OrdMap.get_lazy(ord_map, :a, expensive_fun) &quot;Ant&quot; iex&gt; A.OrdMap.get_lazy(ord_map, :z, expensive_fun) &quot;Zebra&quot;","ref":"A.OrdMap.html#get_lazy/3","title":"A.OrdMap.get_lazy/3","type":"function"},{"doc":"Returns whether the given key exists in ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.has_key?(ord_map, :a) true iex&gt; A.OrdMap.has_key?(ord_map, :d) false","ref":"A.OrdMap.html#has_key?/2","title":"A.OrdMap.has_key?/2","type":"function"},{"doc":"Returns all keys from ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.OrdMap.keys(ord_map) [:b, :c, :a]","ref":"A.OrdMap.html#keys/1","title":"A.OrdMap.keys/1","type":"function"},{"doc":"Finds the last {key, value} pair in ord_map.Returns a {key, value} tuple if ord_map is non-empty, or nil else. Can be accessed efficiently due to the underlying tree.Examplesiex&gt; A.OrdMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.OrdMap.last() {:c, &quot;C&quot;} iex&gt; A.OrdMap.new([]) |&gt; A.OrdMap.last() nil","ref":"A.OrdMap.html#last/1","title":"A.OrdMap.last/1","type":"function"},{"doc":"Merges two ordered maps into one.All keys in ord_map2 will be added to ord_map1, overriding any existing one (i.e., the keys in ord_map2 &quot;have precedence&quot; over the ones in ord_map1).Examplesiex&gt; A.OrdMap.merge(A.OrdMap.new(%{a: 1, b: 2}), A.OrdMap.new(%{a: 3, d: 4})) #A&lt;ord(%{a: 3, b: 2, d: 4})&gt;","ref":"A.OrdMap.html#merge/2","title":"A.OrdMap.merge/2","type":"function"},{"doc":"Returns a new empty ordered map.Examplesiex&gt; A.OrdMap.new() #A&lt;ord(%{})&gt;","ref":"A.OrdMap.html#new/0","title":"A.OrdMap.new/0","type":"function"},{"doc":"Creates an ordered map from an enumerable.Preserves the original order of keys. Duplicated keys are removed; the latest one prevails.Examplesiex&gt; A.OrdMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;) #A&lt;ord(%{b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, a: &quot;Antelope&quot;) #A&lt;ord(%{b: &quot;Buffalo&quot;, a: &quot;Antelope&quot;})&gt;","ref":"A.OrdMap.html#new/1","title":"A.OrdMap.new/1","type":"function"},{"doc":"Creates an ordered map from an enumerable via the given transform function.Preserves the original order of keys. Duplicated keys are removed; the latest one prevails.Examplesiex&gt; A.OrdMap.new([:a, :b], fn x -&gt; {x, x} end) #A&lt;ord(%{a: :a, b: :b})&gt;","ref":"A.OrdMap.html#new/2","title":"A.OrdMap.new/2","type":"function"},{"doc":"Returns the value for key and the updated ordered map without key.If key is present in the ordered map with a value value, {value, new_ord_map} is returned. If key is not present in the ordered map, {default, ord_map} is returned.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;Bat&quot;, updated} = A.OrdMap.pop(ord_map, :b) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; {nil, updated} = A.OrdMap.pop(ord_map, :z) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt; iex&gt; {&quot;Z&quot;, updated} = A.OrdMap.pop(ord_map, :z, &quot;Z&quot;) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#pop/3","title":"A.OrdMap.pop/3","type":"function"},{"doc":"Returns the value for key and the updated ordered map without key.Behaves the same as pop/3 but raises if key is not present in ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;Bat&quot;, updated} = A.OrdMap.pop!(ord_map, :b) iex&gt; updated #A&lt;ord(%{a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.pop!(ord_map, :z) ** (KeyError) key :z not found in: #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#pop!/2","title":"A.OrdMap.pop!/2","type":"function"},{"doc":"Finds and pops the first {key, value} pair in ord_map.Returns a {key, value, new_tree} tuple for non-empty maps, nil for empty mapsExamplesiex&gt; ord_map = A.OrdMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) #A&lt;ord(%{b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;})&gt; iex&gt; {:b, &quot;B&quot;, updated} = A.OrdMap.pop_first(ord_map) iex&gt; updated #A&lt;ord(%{d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;})&gt; iex&gt; A.OrdMap.new() |&gt; A.OrdMap.pop_first() nil","ref":"A.OrdMap.html#pop_first/1","title":"A.OrdMap.pop_first/1","type":"function"},{"doc":"Finds and pops the last {key, value} pair in ord_map.Returns a {key, value, new_tree} tuple for non-empty maps, nil for empty mapsExamplesiex&gt; ord_map = A.OrdMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) #A&lt;ord(%{b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;})&gt; iex&gt; {:c, &quot;C&quot;, updated} = A.OrdMap.pop_last(ord_map) iex&gt; updated #A&lt;ord(%{b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;})&gt; iex&gt; A.OrdMap.new() |&gt; A.OrdMap.pop_last() nil","ref":"A.OrdMap.html#pop_last/1","title":"A.OrdMap.pop_last/1","type":"function"},{"doc":"Lazily returns and removes the value associated with key in ord_map.If key is present in ord_map, it returns {value, new_map} where value is the value of the key and new_map is the result of removing key from ord_map. If key is not present in ord_map, {fun_result, ord_map} is returned, where fun_result is the result of applying fun.This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; {&quot;Ant&quot;, updated} = A.OrdMap.pop_lazy(ord_map, :a, expensive_fun) iex&gt; updated #A&lt;ord(%{b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt; iex&gt; {&quot;Zebra&quot;, not_updated} = A.OrdMap.pop_lazy(ord_map, :z, expensive_fun) iex&gt; not_updated #A&lt;ord(%{b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#pop_lazy/3","title":"A.OrdMap.pop_lazy/3","type":"function"},{"doc":"Puts the given value under key in ord_map.If the key does exist, it overwrites the existing value without changing its current location.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.put(ord_map, :b, &quot;Buffalo&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.put(ord_map, :d, &quot;Dinosaur&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;})&gt;","ref":"A.OrdMap.html#put/3","title":"A.OrdMap.put/3","type":"function"},{"doc":"Puts the given value under key unless the entry key already exists in ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.put_new(ord_map, :a, &quot;Ant&quot;) #A&lt;ord(%{b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;})&gt; iex&gt; A.OrdMap.put_new(ord_map, :b, &quot;Buffalo&quot;) #A&lt;ord(%{b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#put_new/3","title":"A.OrdMap.put_new/3","type":"function"},{"doc":"Evaluates fun and puts the result under key in ord_map unless key is already present.This function is useful in case you want to compute the value to put under key only if key is not already present, as for example, when the value is expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Ant&quot; end iex&gt; A.OrdMap.put_new_lazy(ord_map, :a, expensive_fun) #A&lt;ord(%{b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;})&gt; iex&gt; A.OrdMap.put_new_lazy(ord_map, :b, expensive_fun) #A&lt;ord(%{b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#put_new_lazy/3","title":"A.OrdMap.put_new_lazy/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.replace(ord_map, :b, &quot;Buffalo&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.replace(ord_map, :d, &quot;Dinosaur&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#replace/3","title":"A.OrdMap.replace/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map.If key is not present in ord_map, a KeyError exception is raised.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.replace!(ord_map, :b, &quot;Buffalo&quot;) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.replace!(ord_map, :d, &quot;Dinosaur&quot;) ** (KeyError) key :d not found in: #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#replace!/3","title":"A.OrdMap.replace!/3","type":"function"},{"doc":"Returns all keys from ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.size(ord_map) 3 iex&gt; A.OrdMap.size(A.OrdMap.new()) 0","ref":"A.OrdMap.html#size/1","title":"A.OrdMap.size/1","type":"function"},{"doc":"Returns a new ordered map with all the key-value pairs in ord_map where the key is in keys.If keys contains keys that are not in ord_map, they're simply ignored. Respects the order of the keys list.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.take(ord_map, [:c, :e, :a]) #A&lt;ord(%{c: &quot;Cat&quot;, a: &quot;Ant&quot;})&gt;","ref":"A.OrdMap.html#take/2","title":"A.OrdMap.take/2","type":"function"},{"doc":"Returns all values from ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.OrdMap.to_list(ord_map) [b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;]","ref":"A.OrdMap.html#to_list/1","title":"A.OrdMap.to_list/1","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.update(ord_map, :b, &quot;N/A&quot;, &amp;String.upcase/1) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;BAT&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.update(ord_map, :z, &quot;N/A&quot;, &amp;String.upcase/1) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, z: &quot;N/A&quot;})&gt;","ref":"A.OrdMap.html#update/4","title":"A.OrdMap.update/4","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map.If key is not present in ord_map, a KeyError exception is raised.Examplesiex&gt; ord_map = A.OrdMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.OrdMap.update!(ord_map, :b, &amp;String.upcase/1) #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;BAT&quot;, c: &quot;Cat&quot;})&gt; iex&gt; A.OrdMap.update!(ord_map, :d, &amp;String.upcase/1) ** (KeyError) key :d not found in: #A&lt;ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})&gt;","ref":"A.OrdMap.html#update!/3","title":"A.OrdMap.update!/3","type":"function"},{"doc":"Returns all values from ord_map.Examplesiex&gt; ord_map = A.OrdMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.OrdMap.values(ord_map) [&quot;Bat&quot;, &quot;Cat&quot;, &quot;Ant&quot;]","ref":"A.OrdMap.html#values/1","title":"A.OrdMap.values/1","type":"function"},{"doc":"","ref":"A.OrdMap.html#t:key/0","title":"A.OrdMap.key/0","type":"type"},{"doc":"","ref":"A.OrdMap.html#t:t/0","title":"A.OrdMap.t/0","type":"opaque"},{"doc":"","ref":"A.OrdMap.html#t:t/2","title":"A.OrdMap.t/2","type":"opaque"},{"doc":"","ref":"A.OrdMap.html#t:value/0","title":"A.OrdMap.value/0","type":"type"},{"doc":"Convenience helpers for working with {atom, value} tuples without breaking the pipe.","ref":"A.Pair.html","title":"A.Pair","type":"module"},{"doc":"Unwraps an idiomatic {atom, value} tuple when the atom is what is being expected. Convenient for not breaking the pipe.Examplesiex&gt; {:ok, 55} |&gt; A.Pair.unwrap!(:ok) 55 iex&gt; :error |&gt; A.Pair.unwrap!(:ok) ** (ArgumentError) unwrap!/2 expected {:ok, _}, got: :error","ref":"A.Pair.html#unwrap!/2","title":"A.Pair.unwrap!/2","type":"function"},{"doc":"Wraps the value as an idiomatic {atom, value} tuple. Convenient for not breaking the pipe.Examplesiex&gt; 55 |&gt; A.Pair.wrap(:ok) {:ok, 55} iex&gt; %{a: 5} |&gt; Map.update!(:a, &amp; &amp;1 + 1) |&gt; A.Pair.wrap(:no_reply) {:no_reply, %{a: 6}}","ref":"A.Pair.html#wrap/2","title":"A.Pair.wrap/2","type":"function"},{"doc":"A Red-Black tree implementation of a map. It keeps keys sorted in ascending order.Erlang's :gb_trees offer a similar functionality. However A.RBMap:is fasteris easier to use: pipe-friendliness, Access behaviour, Enum / Inspect / Collectable protocolsrebalances the tree on deletion unlike :gb_treesoptionally implements the Jason.Encoder protocol if Jason is installedExamplesA.RBMap offers the same API as Map :iex&gt; rb_map = A.RBMap.new([b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;]) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.get(rb_map, :c) &quot;Cat&quot; iex&gt; A.RBMap.put(rb_map, :d, &quot;Dinosaur&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;}&gt; iex&gt; A.RBMap.delete(rb_map, :b) #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt; iex&gt; Enum.to_list(rb_map) [a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;] iex&gt; [c: &quot;Cat&quot;, b: &quot;Buffalo&quot;] |&gt; Enum.into(A.RBMap.new([a: &quot;Ant&quot;, b: &quot;Bat&quot;, d: &quot;Dinosaur&quot;])) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;}&gt;Tree-specific functionsDue to its sorted nature, A.RBMap also offers some extra methods not present in Map, like:first/1 and last/1 to efficiently retrieve the first (smallest) / last (largest) key-value pairpop_first/1 and pop_last/1 to efficiently pop the first (smallest) / last (largest) key-value pairfoldl/3 and foldr/3 to efficiently fold (reduce) from left-to-right or right-to-leftExamples:iex&gt; rb_map = A.RBMap.new(%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;, 3 =&gt; &quot;三&quot;}) iex&gt; A.RBMap.first(rb_map) {1, &quot;一&quot;} iex&gt; {3, &quot;三&quot;, updated} = A.RBMap.pop_last(rb_map) iex&gt; updated #A.RBMap&lt;%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;}&gt; iex&gt; A.RBMap.foldr(rb_map, [], fn {_key, value}, acc -&gt; [value | acc] end) [&quot;一&quot;, &quot;二&quot;, &quot;三&quot;]Access behaviourA.RBMap implements the Access behaviour.iex&gt; rb_map = A.RBMap.new([b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;]) iex&gt; rb_map[:a] &quot;Ant&quot; iex&gt; put_in(rb_map[:b], &quot;Buffalo&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; put_in(rb_map[:d], &quot;Dinosaur&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;}&gt; iex&gt; {&quot;Cat&quot;, updated} = pop_in(rb_map[:c]) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;}&gt;With Jasoniex&gt; A.RBMap.new(%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;, 11 =&gt; &quot;十一&quot;}) |&gt; Jason.encode!() &quot;{\\&quot;1\\&quot;:\\&quot;一\\&quot;,\\&quot;2\\&quot;:\\&quot;二\\&quot;,\\&quot;11\\&quot;:\\&quot;十一\\&quot;}&quot;It also preserves the key order.Limitations: pattern-matching and equalityLike :gb_trees, A.RBMaps face two strong limitations:pattern-matching on key-values like maps is NOT POSSIBLEcomparisons based on ==/2, ===/2 or the pin operator ^ are UNRELIABLEIn Elixir, pattern-matching and equality for structs work based on their internal representation. While this is a pragmatic design choice that simplifies the language, it means that we cannot rededine how they work for custom data structures.Tree-based maps that are semantically equal (same key-value pairs in the same order) might be considered non-equal when comparing their internals, because there is not a unique way of representing one same map.A.RBMap.equal?/2 should be used instead:iex&gt; rb_map1 = A.RBMap.new([a: &quot;Ant&quot;, b: &quot;Bat&quot;]) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;}&gt; iex&gt; rb_map2 = A.RBMap.new([b: &quot;Bat&quot;, a: &quot;Ant&quot;]) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;}&gt; iex&gt; rb_map1 == rb_map2 false iex&gt; A.RBMap.equal?(rb_map1, rb_map2) true iex&gt; match?(^rb_map1, rb_map2) falseAn A.RBMap is represented internally using the %A.RBMap{} struct. This struct can be used whenever there's a need to pattern match on something being an A.RBMap:iex&gt; match?(%A.RBMap{}, A.RBMap.new(a: &quot;Ant&quot;)) trueNote, however, than A.RBMap is an opaque type: its struct internal fields must not be accessed directly.Note about numbersUnlike regular maps, A.RBMaps only uses ordering for key comparisons, not strict comparisons. Integers and floats are indistiguinshable as keys.iex&gt; %{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;} |&gt; Map.fetch(2.0) :error iex&gt; A.RBMap.new(%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;}) |&gt; A.RBMap.fetch(2.0) {:ok, &quot;二&quot;}Erlang's :gb_trees module works the same.Difference with A.OrdMapA.OrdMap keeps track of key insertion orderA.RBMap keeps keys sorted in ascending order whatever the insertion order isUnderlying Red-Black Tree implementationThe underlying red-black tree implementation is available in A.RBTree and is used in other modules such as A.RBSet, A.OrdMap as well. The algorithm detail is described in its documentation.","ref":"A.RBMap.html","title":"A.RBMap","type":"module"},{"doc":"Deletes the entry in rb_map for a specific key.If the key does not exist, returns rb_map unchanged.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.delete(rb_map, :b) #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.delete(rb_map, :z) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#delete/2","title":"A.RBMap.delete/2","type":"function"},{"doc":"Drops the given keys from rb_map.If keys contains keys that are not in rb_map, they're simply ignored.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.drop(rb_map, [:b, :d]) #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#drop/2","title":"A.RBMap.drop/2","type":"function"},{"doc":"Checks if two maps are equal.The comparison between keys is done using ==/2, the comparison between values with strict equality ===/2.Examplesiex&gt; A.RBMap.equal?(A.RBMap.new(a: 1, b: 2), A.RBMap.new(b: 2, a: 1)) true iex&gt; A.RBMap.equal?(A.RBMap.new([{1, &quot;一&quot;}, {2, &quot;二&quot;}]), A.RBMap.new([{1, &quot;一&quot;}, {2, &quot;二&quot;}])) true iex&gt; A.RBMap.equal?(A.RBMap.new(a: 1, b: 2), A.RBMap.new(a: 3, b: 2)) false iex&gt; A.RBMap.equal?(A.RBMap.new(a: 1, b: 2), A.RBMap.new(a: 1.0, b: 2.0)) false","ref":"A.RBMap.html#equal?/2","title":"A.RBMap.equal?/2","type":"function"},{"doc":"Fetches the value for a specific key and returns it in a ok-tuple. If the key does not exist, returns :error.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;) iex&gt; A.RBMap.fetch(rb_map, :c) {:ok, &quot;C&quot;} iex&gt; A.RBMap.fetch(rb_map, :z) :error","ref":"A.RBMap.html#fetch/2","title":"A.RBMap.fetch/2","type":"function"},{"doc":"Fetches the value for a specific key and returns it in a ok-tuple. If the key does not exist, returns :error.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;) iex&gt; A.RBMap.fetch!(rb_map, :c) &quot;C&quot; iex&gt; A.RBMap.fetch!(rb_map, :z) ** (KeyError) key :z not found in: #A.RBMap&lt;%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}&gt;","ref":"A.RBMap.html#fetch!/2","title":"A.RBMap.fetch!/2","type":"function"},{"doc":"Finds the {key, value} pair corresponding to the smallest key in rb_map. Returns nil for empty maps.This is very efficient and can be done in O(log(n)). It should be preferred over Enum.min/3.Examplesiex&gt; A.RBMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.RBMap.first() {:a, &quot;A&quot;} iex&gt; A.RBMap.new([]) |&gt; A.RBMap.first() nil","ref":"A.RBMap.html#first/1","title":"A.RBMap.first/1","type":"function"},{"doc":"Folds (reduces) the given map from the right with a function. Requires an accumulator.Examplesiex&gt; rb_map = A.RBMap.new([b: 22, a: 11, c: 33]) iex&gt; A.RBMap.foldl(rb_map, 0, fn {_key, value}, acc -&gt; value + acc end) 66 iex&gt; A.RBMap.foldl(rb_map, [], fn {key, value}, acc -&gt; [{key, value * 2} | acc] end) [c: 66, b: 44, a: 22]","ref":"A.RBMap.html#foldl/3","title":"A.RBMap.foldl/3","type":"function"},{"doc":"Folds (reduces) the given map from the right with a function. Requires an accumulator.Unlike linked lists, this is as efficient as foldl/3. This can typically save a call to Enum.reverse/1 on the result when building a list.Examplesiex&gt; rb_map = A.RBMap.new([b: 22, a: 11, c: 33]) iex&gt; A.RBMap.foldr(rb_map, 0, fn {_key, value}, acc -&gt; value + acc end) 66 iex&gt; A.RBMap.foldr(rb_map, [], fn {key, value}, acc -&gt; [{key, value * 2} | acc] end) [a: 22, b: 44, c: 66]","ref":"A.RBMap.html#foldr/3","title":"A.RBMap.foldr/3","type":"function"},{"doc":"Converts a struct to a A.RBMap.It accepts the struct module or a struct itself and simply removes the __struct__ field from the given struct or from a new struct generated from the given module.Exampledefmodule User do defstruct [:name, :age] end A.RBMap.from_struct(User) #A.RBMap&lt;%{age: nil, name: nil}&gt; A.RBMap.from_struct(%User{name: &quot;john&quot;, age: 44}) #A.RBMap&lt;%{name: &quot;john&quot;}&gt;","ref":"A.RBMap.html#from_struct/1","title":"A.RBMap.from_struct/1","type":"function"},{"doc":"Gets the value for a specific key in rb_map.If key is present in rb_map then its value value is returned. Otherwise, default is returned.If default is not provided, nil is used.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.get(rb_map, :a) &quot;Ant&quot; iex&gt; A.RBMap.get(rb_map, :z) nil iex&gt; A.RBMap.get(rb_map, :z, &quot;Zebra&quot;) &quot;Zebra&quot;","ref":"A.RBMap.html#get/3","title":"A.RBMap.get/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass.Mirrors Map.get_and_update/3, see its documentation.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;bat&quot;, updated} = A.RBMap.get_and_update(rb_map, :b, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Buffalo&quot;} ...&gt; end) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; {nil, updated} = A.RBMap.get_and_update(rb_map, :z, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Zebra&quot;} ...&gt; end) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, z: &quot;Zebra&quot;}&gt; iex&gt; {&quot;Bat&quot;, updated} = A.RBMap.get_and_update(rb_map, :b, fn _ -&gt; :pop end) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt; iex&gt; {nil, updated} = A.RBMap.get_and_update(rb_map, :z, fn _ -&gt; :pop end) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#get_and_update/3","title":"A.RBMap.get_and_update/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass.Mirrors Map.get_and_update!/3, see its documentation.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;bat&quot;, updated} = A.RBMap.get_and_update!(rb_map, :b, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Buffalo&quot;} ...&gt; end) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.get_and_update!(rb_map, :z, fn current_value -&gt; ...&gt; {current_value &amp;&amp; String.downcase(current_value), &quot;Zebra&quot;} ...&gt; end) ** (KeyError) key :z not found in: #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#get_and_update!/3","title":"A.RBMap.get_and_update!/3","type":"function"},{"doc":"Gets the value for a specific key in rb_map.If key is present in rb_map then its value value is returned. Otherwise, fun is evaluated and its result is returned.This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; A.RBMap.get_lazy(rb_map, :a, expensive_fun) &quot;Ant&quot; iex&gt; A.RBMap.get_lazy(rb_map, :z, expensive_fun) &quot;Zebra&quot;","ref":"A.RBMap.html#get_lazy/3","title":"A.RBMap.get_lazy/3","type":"function"},{"doc":"Returns whether the given key exists in rb_map.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.has_key?(rb_map, :a) true iex&gt; A.RBMap.has_key?(rb_map, :d) false iex&gt; A.RBMap.has_key?(A.RBMap.new(%{1.0 =&gt; &quot;uno&quot;}), 1) true","ref":"A.RBMap.html#has_key?/2","title":"A.RBMap.has_key?/2","type":"function"},{"doc":"","ref":"A.RBMap.html#iterator/1","title":"A.RBMap.iterator/1","type":"function"},{"doc":"Returns all keys from rb_map.Examplesiex&gt; rb_map = A.RBMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.RBMap.keys(rb_map) [:a, :b, :c]","ref":"A.RBMap.html#keys/1","title":"A.RBMap.keys/1","type":"function"},{"doc":"Finds the {key, value} pair corresponding to the largest key in rb_map. Returns nil for empty maps.This is very efficient and can be done in O(log(n)). It should be preferred over Enum.max/3.Examplesiex&gt; A.RBMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.RBMap.last() {:d, &quot;D&quot;} iex&gt; A.RBMap.new([]) |&gt; A.RBMap.last() nil","ref":"A.RBMap.html#last/1","title":"A.RBMap.last/1","type":"function"},{"doc":"Merges two maps into one.All keys in rb_map2 will be added to rb_map1, overriding any existing one (i.e., the keys in rb_map2 &quot;have precedence&quot; over the ones in rb_map1).Examplesiex&gt; A.RBMap.merge(A.RBMap.new(%{a: 1, b: 2}), A.RBMap.new(%{a: 3, d: 4})) #A.RBMap&lt;%{a: 3, b: 2, d: 4}&gt;","ref":"A.RBMap.html#merge/2","title":"A.RBMap.merge/2","type":"function"},{"doc":"Returns a new empty map.Examplesiex&gt; A.RBMap.new() #A.RBMap&lt;%{}&gt;","ref":"A.RBMap.html#new/0","title":"A.RBMap.new/0","type":"function"},{"doc":"Creates a map from an enumerable.Keys are sorted upon insertion, and duplicated keys are removed; the latest one prevails.Examplesiex&gt; A.RBMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.new(b: &quot;Bat&quot;, a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, a: &quot;Antelope&quot;) #A.RBMap&lt;%{a: &quot;Antelope&quot;, b: &quot;Buffalo&quot;}&gt;","ref":"A.RBMap.html#new/1","title":"A.RBMap.new/1","type":"function"},{"doc":"Creates a map from an enumerable via the given transform function.Duplicated keys are removed; the latest one prevails.Examplesiex&gt; A.RBMap.new([:a, :b], fn x -&gt; {x, x} end) #A.RBMap&lt;%{a: :a, b: :b}&gt;","ref":"A.RBMap.html#new/2","title":"A.RBMap.new/2","type":"function"},{"doc":"See A.RBTree.next/1.","ref":"A.RBMap.html#next/1","title":"A.RBMap.next/1","type":"function"},{"doc":"Returns the value for key and the updated map without key.If key is present in the map with a value value, {value, new_rb_map} is returned. If key is not present in the map, {default, rb_map} is returned.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;Bat&quot;, updated} = A.RBMap.pop(rb_map, :b) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt; iex&gt; {nil, updated} = A.RBMap.pop(rb_map, :z) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt; iex&gt; {&quot;Z&quot;, updated} = A.RBMap.pop(rb_map, :z, &quot;Z&quot;) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#pop/3","title":"A.RBMap.pop/3","type":"function"},{"doc":"Returns the value for key and the updated map without key.Behaves the same as pop/3 but raises if key is not present in rb_map.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; {&quot;Bat&quot;, updated} = A.RBMap.pop!(rb_map, :b) iex&gt; updated #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.pop!(rb_map, :z) ** (KeyError) key :z not found in: #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#pop!/2","title":"A.RBMap.pop!/2","type":"function"},{"doc":"Finds and pops the {key, value} pair corresponding to the smallest key in rb_map.Returns a {key, value, new_tree} tuple for non-empty maps, nil for empty mapsExamplesiex&gt; rb_map = A.RBMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) #A.RBMap&lt;%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;, d: &quot;D&quot;}&gt; iex&gt; {:a, &quot;A&quot;, updated} = A.RBMap.pop_first(rb_map) iex&gt; updated #A.RBMap&lt;%{b: &quot;B&quot;, c: &quot;C&quot;, d: &quot;D&quot;}&gt; iex&gt; A.RBMap.new() |&gt; A.RBMap.pop_first() nil","ref":"A.RBMap.html#pop_first/1","title":"A.RBMap.pop_first/1","type":"function"},{"doc":"Finds and pops the {key, value} pair corresponding to the largest key in rb_map.Returns a {key, value, new_tree} tuple for non-empty maps, nil for empty mapsExamplesiex&gt; rb_map = A.RBMap.new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) #A.RBMap&lt;%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;, d: &quot;D&quot;}&gt; iex&gt; {:d, &quot;D&quot;, updated} = A.RBMap.pop_last(rb_map) iex&gt; updated #A.RBMap&lt;%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}&gt; iex&gt; A.RBMap.new() |&gt; A.RBMap.pop_last() nil","ref":"A.RBMap.html#pop_last/1","title":"A.RBMap.pop_last/1","type":"function"},{"doc":"Lazily returns and removes the value associated with key in rb_map.If key is present in rb_map, it returns {value, new_map} where value is the value of the key and new_map is the result of removing key from rb_map. If key is not present in rb_map, {fun_result, rB_map} is returned, where fun_result is the result of applying fun.This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; {&quot;Ant&quot;, updated} = A.RBMap.pop_lazy(rb_map, :a, expensive_fun) iex&gt; updated #A.RBMap&lt;%{b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt; iex&gt; {&quot;Zebra&quot;, not_updated} = A.RBMap.pop_lazy(rb_map, :z, expensive_fun) iex&gt; not_updated #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#pop_lazy/3","title":"A.RBMap.pop_lazy/3","type":"function"},{"doc":"Puts the given value under key in rb_map.If the key does exist, it overwrites the existing value.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.put(rb_map, :b, &quot;Buffalo&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.put(rb_map, :d, &quot;Dinosaur&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;, d: &quot;Dinosaur&quot;}&gt;","ref":"A.RBMap.html#put/3","title":"A.RBMap.put/3","type":"function"},{"doc":"Puts the given value under key unless the entry key already exists in rb_map.Examplesiex&gt; rb_map = A.RBMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.put_new(rb_map, :a, &quot;Ant&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.put_new(rb_map, :b, &quot;Buffalo&quot;) #A.RBMap&lt;%{b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#put_new/3","title":"A.RBMap.put_new/3","type":"function"},{"doc":"Evaluates fun and puts the result under key in rb_map unless key is already present.This function is useful in case you want to compute the value to put under key only if key is not already present, as for example, when the value is expensive to calculate or generally difficult to setup and teardown again.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, c: &quot;Cat&quot;) iex&gt; expensive_fun = fn -&gt; &quot;Buffalo&quot; end iex&gt; A.RBMap.put_new_lazy(rb_map, :b, expensive_fun) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.put_new_lazy(rb_map, :a, expensive_fun) #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#put_new_lazy/3","title":"A.RBMap.put_new_lazy/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in rb_map.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.replace(rb_map, :b, &quot;Buffalo&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.replace(rb_map, :d, &quot;Dinosaur&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#replace/3","title":"A.RBMap.replace/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in rb_map.If key is not present in rb_map, a KeyError exception is raised.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.replace!(rb_map, :b, &quot;Buffalo&quot;) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.replace!(rb_map, :d, &quot;Dinosaur&quot;) ** (KeyError) key :d not found in: #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#replace!/3","title":"A.RBMap.replace!/3","type":"function"},{"doc":"Returns the number of keys in rb_map.Examplesiex&gt; A.RBMap.size(A.RBMap.new(a: 1, b: 2, c: 3)) 3 iex&gt; A.RBMap.size(A.RBMap.new(a: 1, a: 2, a: 3)) 1","ref":"A.RBMap.html#size/1","title":"A.RBMap.size/1","type":"function"},{"doc":"Returns a new map with all the key-value pairs in rb_map where the key is in keys.If keys contains keys that are not in rb_map, they're simply ignored.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.take(rb_map, [:c, :e, :a]) #A.RBMap&lt;%{a: &quot;Ant&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#take/2","title":"A.RBMap.take/2","type":"function"},{"doc":"Returns all values from rb_map.Examplesiex&gt; rb_map = A.RBMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.RBMap.to_list(rb_map) [a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;]","ref":"A.RBMap.html#to_list/1","title":"A.RBMap.to_list/1","type":"function"},{"doc":"Updates the key in rb_map with the given function.If key is present in rb_map then the existing value is passed to fun and its result is used as the updated value of key. If key is not present in rb_map, default is inserted as the value of key. The default value will not be passed through the update function.Examplesiex&gt; rb_map = A.RBMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt;A.RBMap.update(rb_map, :b, &quot;N/A&quot;, &amp;String.upcase/1) #A.RBMap&lt;%{b: &quot;BAT&quot;, c: &quot;Cat&quot;}&gt; iex&gt;A.RBMap.update(rb_map, :a, &quot;N/A&quot;, &amp;String.upcase/1) #A.RBMap&lt;%{a: &quot;N/A&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#update/4","title":"A.RBMap.update/4","type":"function"},{"doc":"Puts a value under key only if the key already exists in rb_map.If key is not present in rb_map, a KeyError exception is raised.Examplesiex&gt; rb_map = A.RBMap.new(a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;) iex&gt; A.RBMap.update!(rb_map, :b, &amp;String.upcase/1) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;BAT&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBMap.update!(rb_map, :d, &amp;String.upcase/1) ** (KeyError) key :d not found in: #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;}&gt;","ref":"A.RBMap.html#update!/3","title":"A.RBMap.update!/3","type":"function"},{"doc":"Returns all values from rb_map.Examplesiex&gt; rb_map = A.RBMap.new(b: &quot;Bat&quot;, c: &quot;Cat&quot;, a: &quot;Ant&quot;) iex&gt; A.RBMap.values(rb_map) [&quot;Ant&quot;, &quot;Bat&quot;, &quot;Cat&quot;]","ref":"A.RBMap.html#values/1","title":"A.RBMap.values/1","type":"function"},{"doc":"","ref":"A.RBMap.html#t:iterator/2","title":"A.RBMap.iterator/2","type":"opaque"},{"doc":"","ref":"A.RBMap.html#t:key/0","title":"A.RBMap.key/0","type":"type"},{"doc":"","ref":"A.RBMap.html#t:t/0","title":"A.RBMap.t/0","type":"opaque"},{"doc":"","ref":"A.RBMap.html#t:t/2","title":"A.RBMap.t/2","type":"opaque"},{"doc":"","ref":"A.RBMap.html#t:value/0","title":"A.RBMap.value/0","type":"type"},{"doc":"A Red-Black tree implementation of a set. It keeps elements sorted in ascending order.It works as a drop-in replacement for the built-in MapSet. Unlike MapSet which does not keep keys in any particular order, A.RBSet stores keys in ascending order.Erlang's :gb_sets offer a similar functionality. However A.RBSet:is fasteris easier to use (pipe-friendliness, Enum / Inspect / Collectable protocols)rebalances the tree on deletion unlike :gb_setsoptionally implements the Jason.Encoder protocol if Jason is installedExamplesA.RBSet offers the same API as MapSet:iex&gt; rb_set = A.RBSet.new([6, 6, 7, 7, 4, 1, 2, 3, 1.0, 5]) #A.RBSet&lt;[1.0, 2, 3, 4, 5, 6, 7]&gt; iex&gt; A.RBSet.member?(rb_set, 2) true iex&gt; A.RBSet.member?(rb_set, 8) false iex&gt; A.RBSet.put(rb_set, 4.25) #A.RBSet&lt;[1.0, 2, 3, 4, 4.25, 5, 6, 7]&gt; iex&gt; A.RBSet.delete(rb_set, 1) #A.RBSet&lt;[2, 3, 4, 5, 6, 7]&gt; iex&gt; A.RBSet.union(rb_set, A.RBSet.new([0, 2, 4, 6, 8])) #A.RBSet&lt;[0, 1.0, 2, 3, 4, 5, 6, 7, 8]&gt; iex&gt; A.RBSet.intersection(rb_set, A.RBSet.new([0, 2, 4, 6, 8])) #A.RBSet&lt;[2, 4, 6]&gt; iex&gt; A.RBSet.difference(rb_set, A.RBSet.new([0, 2, 4, 6, 8])) #A.RBSet&lt;[1.0, 3, 5, 7]&gt; iex&gt; Enum.to_list(rb_set) [1.0, 2, 3, 4, 5, 6, 7] iex&gt; [0, 1, 1.1, 2.2, 3.3] |&gt; Enum.into(rb_set) #A.RBSet&lt;[0, 1, 1.1, 2, 2.2, 3, 3.3, 4, 5, 6, 7]&gt;Like for MapSets, elements in a set don't have to be of the same type:iex&gt; A.RBSet.new([1, :two, {&quot;three&quot;}]) #A.RBSet&lt;[1, :two, {&quot;three&quot;}]&gt;Tree-specific functionsDue to its sorted nature, A.RBSet also offers some extra methods not present in MapSet, like:first/1 and last/1 to efficiently retrieve the first (smallest) / last (largest) elementpop_first/1 and pop_last/1 to efficiently pop the first (smallest) / last (largest) elementfoldl/3 and foldr/3 to efficiently fold (reduce) from left-to-right or right-to-leftExamples:iex&gt; rb_set = A.RBSet.new([8, 6, 0, 4, 2, 2, 2]) iex&gt; A.RBSet.last(rb_set) 8 iex&gt; {0, updated} = A.RBSet.pop_first(rb_set) iex&gt; updated #A.RBSet&lt;[2, 4, 6, 8]&gt; iex&gt; A.RBSet.foldr(rb_set, [], fn value, acc -&gt; [value + 1 | acc] end) [1, 3, 5, 7, 9]With Jasoniex&gt; A.RBSet.new([6, 6, 7, 7, 4, 1, 2, 3, 1.0, 5]) |&gt; Jason.encode!() &quot;[1.0,2,3,4,5,6,7]&quot;It also preserves the element order.Limitations: equalityLike :gb_sets, A.RBSet comparisons based on ==/2, ===/2 or the pin operator ^ are UNRELIABLE.In Elixir, pattern-matching and equality for structs work based on their internal representation. While this is a pragmatic design choice that simplifies the language, it means that we cannot rededine how they work for custom data structures.Tree-based sets that are semantically equal (same elements in the same order) might be considered non-equal when comparing their internals, because there is not a unique way of representing one same set.A.RBSet.equal?/2 should be used instead:iex&gt; rb_set1 = A.RBSet.new([1, 2]) #A.RBSet&lt;[1, 2]&gt; iex&gt; rb_set2 = A.RBSet.new([2, 1]) #A.RBSet&lt;[1, 2]&gt; iex&gt; rb_set1 == rb_set2 false iex&gt; A.RBSet.equal?(rb_set1, rb_set2) true iex&gt; match?(^rb_set1, rb_set2) falsePattern-matching and opaque typeAn A.RBSet is represented internally using the %A.RBSet{} struct. This struct can be used whenever there's a need to pattern match on something being an A.RBSet:iex&gt; match?(%A.RBSet{}, A.RBSet.new()) trueNote, however, than A.RBSet is an opaque type: its struct internal fields must not be accessed directly.Use the functions in this module to perform operations on A.RBSets, or the Enum module.Note about numbersUnlike MapSets, A.RBSets only uses ordering for element comparisons, not strict comparisons. Integers and floats are indistiguinshable as elements.iex&gt; MapSet.new([1, 2, 3]) |&gt; MapSet.member?(2.0) false iex&gt; A.RBSet.new([1, 2, 3]) |&gt; A.RBSet.member?(2.0) trueErlang's :gb_sets module works the same.Underlying Red-Black Tree implementationThe underlying red-black tree implementation is available in A.RBTree and is used in other modules such as A.RBMap, A.OrdMap as well. The algorithm detail is described in its documentation.","ref":"A.RBSet.html","title":"A.RBSet","type":"module"},{"doc":"Deletes value from rb_set.Returns a new set which is a copy of rb_set but without value.Examplesiex&gt; rb_set = A.RBSet.new([1, 2, 3]) iex&gt; A.RBSet.delete(rb_set, 4) #A.RBSet&lt;[1, 2, 3]&gt; iex&gt; A.RBSet.delete(rb_set, 2) #A.RBSet&lt;[1, 3]&gt;","ref":"A.RBSet.html#delete/2","title":"A.RBSet.delete/2","type":"function"},{"doc":"Returns a set that is rb_set1 without the members of rb_set2.Examplesiex&gt; A.RBSet.difference(A.RBSet.new([1, 2]), A.RBSet.new([2, 3, 4])) #A.RBSet&lt;[1]&gt;","ref":"A.RBSet.html#difference/2","title":"A.RBSet.difference/2","type":"function"},{"doc":"Checks if rb_set1 and rb_set2 have no members in common.Examplesiex&gt; A.RBSet.disjoint?(A.RBSet.new([1, 2]), A.RBSet.new([3, 4])) true iex&gt; A.RBSet.disjoint?(A.RBSet.new([1, 2]), A.RBSet.new([2, 3])) false","ref":"A.RBSet.html#disjoint?/2","title":"A.RBSet.disjoint?/2","type":"function"},{"doc":"Checks if two sets are equal.The comparison between elements is done using ==/2, not strict equality ===/2.Examplesiex&gt; A.RBSet.equal?(A.RBSet.new([1, 2]), A.RBSet.new([2, 1, 1])) true iex&gt; A.RBSet.equal?(A.RBSet.new([1.0, 2.0]), A.RBSet.new([2, 1, 1])) true iex&gt; A.RBSet.equal?(A.RBSet.new([1, 2]), A.RBSet.new([3, 4])) false","ref":"A.RBSet.html#equal?/2","title":"A.RBSet.equal?/2","type":"function"},{"doc":"Finds the smallest element in the set. Returns nil for empty sets.This is very efficient and can be done in O(log(n)). It should be preferred over Enum.min/3.Examplesiex&gt; A.RBSet.new([4, 2, 3]) |&gt; A.RBSet.first() 2 iex&gt; A.RBSet.new() |&gt; A.RBSet.first() nil","ref":"A.RBSet.html#first/1","title":"A.RBSet.first/1","type":"function"},{"doc":"Folds (reduces) the given set from the right with a function. Requires an accumulator.Examplesiex&gt; A.RBSet.new([22, 11, 33]) |&gt; A.RBSet.foldl(0, &amp;+/2) 66 iex&gt; A.RBSet.new([22, 11, 33]) |&gt; A.RBSet.foldl([], &amp;([2 * &amp;1 | &amp;2])) [66, 44, 22]","ref":"A.RBSet.html#foldl/3","title":"A.RBSet.foldl/3","type":"function"},{"doc":"Folds (reduces) the given set from the right with a function. Requires an accumulator.Unlike linked lists, this is as efficient as foldl/3. This can typically save a call to Enum.reverse/1 on the result when building a list.Examplesiex&gt; A.RBSet.new([22, 11, 33]) |&gt; A.RBSet.foldr(0, &amp;+/2) 66 iex&gt; A.RBSet.new([22, 11, 33]) |&gt; A.RBSet.foldr([], &amp;([2 * &amp;1 | &amp;2])) [22, 44, 66]","ref":"A.RBSet.html#foldr/3","title":"A.RBSet.foldr/3","type":"function"},{"doc":"Returns a set containing only members that rb_set1 and rb_set2 have in common.Examplesiex&gt; A.RBSet.intersection(A.RBSet.new([2, 1]), A.RBSet.new([3, 2, 4])) #A.RBSet&lt;[2]&gt; iex&gt; A.RBSet.intersection(A.RBSet.new([2, 1]), A.RBSet.new([3, 4])) #A.RBSet&lt;[]&gt;","ref":"A.RBSet.html#intersection/2","title":"A.RBSet.intersection/2","type":"function"},{"doc":"Finds the largest element in the set. Returns nil for empty sets.This is very efficient and can be done in O(log(n)). It should be preferred over Enum.max/3.Examplesiex&gt; A.RBSet.new([4, 2, 3]) |&gt; A.RBSet.last() 4 iex&gt; A.RBSet.new() |&gt; A.RBSet.last() nil","ref":"A.RBSet.html#last/1","title":"A.RBSet.last/1","type":"function"},{"doc":"Checks if rb_set contains value.Examplesiex&gt; A.RBSet.member?(A.RBSet.new([1, 2, 3]), 2) true iex&gt; A.RBSet.member?(A.RBSet.new([1, 2, 3]), 4) false","ref":"A.RBSet.html#member?/2","title":"A.RBSet.member?/2","type":"function"},{"doc":"Returns a new empty set.Examplesiex&gt; A.RBSet.new() #A.RBSet&lt;[]&gt;","ref":"A.RBSet.html#new/0","title":"A.RBSet.new/0","type":"function"},{"doc":"Creates a set from an enumerable.Examplesiex&gt; A.RBSet.new([:b, :a, 3]) #A.RBSet&lt;[3, :a, :b]&gt; iex&gt; A.RBSet.new([3, 3, 3, 2, 2, 1]) #A.RBSet&lt;[1, 2, 3]&gt;","ref":"A.RBSet.html#new/1","title":"A.RBSet.new/1","type":"function"},{"doc":"Creates a set from an enumerable via the transformation function.Examplesiex&gt; A.RBSet.new([1, 2, 1], fn x -&gt; 2 * x end) #A.RBSet&lt;[2, 4]&gt;","ref":"A.RBSet.html#new/2","title":"A.RBSet.new/2","type":"function"},{"doc":"Removes and returns the smallest element in the set.Returns a {value, new_rb_set} tuple when non-empty, or nil for empty sets.Examplesiex&gt; rb_set = A.RBSet.new([4, 2, 5, 3]) iex&gt; {2, updated} = A.RBSet.pop_first(rb_set) iex&gt; updated #A.RBSet&lt;[3, 4, 5]&gt; iex&gt; A.RBSet.new() |&gt; A.RBSet.pop_first() nil","ref":"A.RBSet.html#pop_first/1","title":"A.RBSet.pop_first/1","type":"function"},{"doc":"Removes and returns the largest element in the set.Returns a {value, new_rb_set} tuple when non-empty, or nil for empty sets.Examplesiex&gt; rb_set = A.RBSet.new([4, 2, 5, 3]) iex&gt; {5, updated} = A.RBSet.pop_last(rb_set) iex&gt; updated #A.RBSet&lt;[2, 3, 4]&gt; iex&gt; A.RBSet.new() |&gt; A.RBSet.pop_last() nil","ref":"A.RBSet.html#pop_last/1","title":"A.RBSet.pop_last/1","type":"function"},{"doc":"Inserts value into rb_set if rb_set doesn't already contain it.Examplesiex&gt; A.RBSet.put(A.RBSet.new([1, 2, 3]), 3) #A.RBSet&lt;[1, 2, 3]&gt; iex&gt; A.RBSet.put(A.RBSet.new([1, 2, 3]), 4) #A.RBSet&lt;[1, 2, 3, 4]&gt;","ref":"A.RBSet.html#put/2","title":"A.RBSet.put/2","type":"function"},{"doc":"Returns the number of elements in rb_set.Examplesiex&gt; A.RBSet.size(A.RBSet.new([1, 2, 3])) 3 iex&gt; A.RBSet.size(A.RBSet.new([1, 1, 1.0])) 1","ref":"A.RBSet.html#size/1","title":"A.RBSet.size/1","type":"function"},{"doc":"Checks if rb_set1's members are all contained in rb_set2.This function checks if rb_set1 is a subset of rb_set2.Examplesiex&gt; A.RBSet.subset?(A.RBSet.new([1, 2]), A.RBSet.new([1, 2, 3])) true iex&gt; A.RBSet.subset?(A.RBSet.new([1, 2, 3]), A.RBSet.new([1, 2])) false","ref":"A.RBSet.html#subset?/2","title":"A.RBSet.subset?/2","type":"function"},{"doc":"Converts rb_set to a list.Examplesiex&gt; A.RBSet.to_list(A.RBSet.new([1, 2, 3])) [1, 2, 3]","ref":"A.RBSet.html#to_list/1","title":"A.RBSet.to_list/1","type":"function"},{"doc":"Returns a set containing all members of rb_set1 and rb_set2.Examplesiex&gt; A.RBSet.union(A.RBSet.new([2, 1]), A.RBSet.new([4, 2, 3])) #A.RBSet&lt;[1, 2, 3, 4]&gt;","ref":"A.RBSet.html#union/2","title":"A.RBSet.union/2","type":"function"},{"doc":"","ref":"A.RBSet.html#t:t/0","title":"A.RBSet.t/0","type":"type"},{"doc":"","ref":"A.RBSet.html#t:t/1","title":"A.RBSet.t/1","type":"opaque"},{"doc":"","ref":"A.RBSet.html#t:value/0","title":"A.RBSet.value/0","type":"type"},{"doc":"A low-level implementation of a Red-Black Tree, used under the hood in A.RBMap, A.RBSet and A.OrdMap.Implementation following Chris Okasaki's &quot;Purely Functional Data Structures&quot;, with the delete method as described in Deletion: The curse of the red-black tree from German and Might.It should perform significantly better than built-in :gb_trees and :gb_sets (see benchmarks).DisclaimerThis module is the low-level implementation behind other data structures, it is NOT meant to be used directly.If you want something ready to use, you should check A.RBMap and A.RBSet (or maybe A.OrdMap).Probably the only case you might be interested in A.RBTree itself is if you want to implement your own data structures on the top of it, or out of curiosity.It implements both a Map API and a Set API, both should not be mixed.Map APIiex&gt; A.RBTree.map_new([]) :E iex&gt; map = A.RBTree.map_new([b: &quot;B&quot;, c: &quot;C&quot;, a: &quot;A&quot;]) {:B, {:R, :E, {:a, &quot;A&quot;}, :E}, {:b, &quot;B&quot;}, {:R, :E, {:c, &quot;C&quot;}, :E}} iex&gt; A.RBTree.map_fetch(map, :c) {:ok, &quot;C&quot;} iex&gt; {:new, _new_map} = A.RBTree.map_insert(map, :bar, &quot;BAR&quot;) {:new, {:B, {:B, {:R, :E, {:a, &quot;A&quot;}, :E}, {:b, &quot;B&quot;}, :E}, {:bar, &quot;BAR&quot;}, {:B, :E, {:c, &quot;C&quot;}, :E}}} iex&gt; {:ok, &quot;B&quot;, _new_map} = A.RBTree.map_pop(map, :b) {:ok, &quot;B&quot;, {:B, {:R, :E, {:a, &quot;A&quot;}, :E}, {:c, &quot;C&quot;}, :E}} iex&gt; A.RBTree.map_pop(map, :bar) :error iex&gt; A.RBTree.map_new([b: &quot;B&quot;, x: &quot;X&quot;, c: &quot;C&quot;, a: &quot;A&quot;]) |&gt; A.RBTree.to_list() [a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;, x: &quot;X&quot;]Set APIiex&gt; A.RBTree.set_new([]) :E iex&gt; set = A.RBTree.set_new([2.0, 3, 2, 1, 3, 3]) {:B, {:R, :E, 1, :E}, 2, {:R, :E, 3, :E}} iex&gt; A.RBTree.set_member?(set, 3) true iex&gt; {:new, _new_set} = A.RBTree.set_insert(set, 2.5) {:new, {:B, {:B, {:R, :E, 1, :E}, 2, :E}, 2.5, {:B, :E, 3, :E}}} iex&gt; {:ok, _new_set} = A.RBTree.set_delete(set, 2) {:ok, {:B, {:R, :E, 1, :E}, 3, :E}} iex&gt; A.RBTree.set_delete(set, 4) :error iex&gt; A.RBTree.set_new([9, 8, 8, 7, 4, 1, 1, 2, 3, 3, 3, 9, 5, 6]) |&gt; A.RBTree.to_list() [1, 2, 3, 4, 5, 6, 7, 8, 9]For the curious reader: more about deletionInsertion is easy enough in an immutable Red-Black Tree, deletion however is pretty tricky. Two implementations have been tried:this approach from Matt MightDeletion: The curse of the red-black tree from Germane and Might1. The Haskell implementation used as a reference has a bug and seems not to be respecting the Red-Black invariant, as suggested here.2. was retained and it was confirmed that the Red-Black invariant was maintained.Finally, a third approach from Kahr's (example Haskell implementation) seems to be faster and might be tried in future iterations.Note about numbersUnlike regular maps, A.RBTrees only uses ordering for key comparisons, meaning integers and floats are indistiguinshable as keys.iex&gt; %{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;} |&gt; Map.fetch(2) {:ok, &quot;二&quot;} iex&gt; %{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;} |&gt; Map.fetch(2.0) :error iex&gt; A.RBTree.map_new(%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;}) |&gt; A.RBTree.map_fetch(2) {:ok, &quot;二&quot;} iex&gt; A.RBTree.map_new(%{1 =&gt; &quot;一&quot;, 2 =&gt; &quot;二&quot;}) |&gt; A.RBTree.map_fetch(2.0) {:ok, &quot;二&quot;}Erlang's :gb_trees module works the same.","ref":"A.RBTree.html","title":"A.RBTree","type":"module"},{"doc":"","ref":"A.RBTree.html#black_height/1","title":"A.RBTree.black_height/1","type":"function"},{"doc":"Checks the red-black invariant is respected:Each tree is either red or black. The root is black. This rule is sometimes omitted. Since the root can always be changed from red to black, but not necessarily vice versa, this rule has little effect on analysis. (All leaves (NIL) are black.) If a tree is red, then both its children are black. Every path from a given tree to any of its descendant NIL trees goes through the same number of black trees.Returns either an {:ok, black_height} tuple if respected and black_height is consistent, or an {:error, reason} tuple if violated.Examplesiex&gt; A.RBTree.check_invariant(:E) {:ok, 0} iex&gt; A.RBTree.check_invariant({:B, :E, {1, nil}, :E}) {:ok, 1} iex&gt; A.RBTree.check_invariant({:R, :E, {1, nil}, :E}) {:error, &quot;No red root allowed&quot;} iex&gt; A.RBTree.check_invariant({:B, {:B, :E, {1, nil}, :E}, {2, nil}, :E}) {:error, &quot;Inconsistent black length&quot;} iex&gt; A.RBTree.check_invariant({:B, {:R, {:R, :E, {1, nil}, :E}, {2, nil}, :E}, {3, nil}, :E}) {:error, &quot;Red tree has red child&quot;}","ref":"A.RBTree.html#check_invariant/1","title":"A.RBTree.check_invariant/1","type":"function"},{"doc":"","ref":"A.RBTree.html#check_invariant!/1","title":"A.RBTree.check_invariant!/1","type":"function"},{"doc":"","ref":"A.RBTree.html#empty/0","title":"A.RBTree.empty/0","type":"function"},{"doc":"Folds (reduces) the given tree from the left with a function. Requires an accumulator.Examplesiex&gt; A.RBTree.set_new([22, 11, 33]) |&gt; A.RBTree.foldl(0, &amp;+/2) 66 iex&gt; A.RBTree.set_new([22, 11, 33]) |&gt; A.RBTree.foldl([], &amp;([2 * &amp;1 | &amp;2])) [66, 44, 22]","ref":"A.RBTree.html#foldl/3","title":"A.RBTree.foldl/3","type":"function"},{"doc":"Folds (reduces) the given tree from the right with a function. Requires an accumulator.Unlike linked lists, this is as efficient as foldl/3. This can typically save a call to Enum.reverse/1 on the result when building a list.Examplesiex&gt; A.RBTree.set_new([22, 11, 33]) |&gt; A.RBTree.foldr(0, &amp;+/2) 66 iex&gt; A.RBTree.set_new([22, 11, 33]) |&gt; A.RBTree.foldr([], &amp;([2 * &amp;1 | &amp;2])) [22, 44, 66]","ref":"A.RBTree.html#foldr/3","title":"A.RBTree.foldr/3","type":"function"},{"doc":"","ref":"A.RBTree.html#height/1","title":"A.RBTree.height/1","type":"function"},{"doc":"Returns an iterator looping on a tree from left-to-right.The resulting iterator should be looped over using next/1.Examplesiex&gt; iterator = A.RBTree.set_new([22, 11]) |&gt; A.RBTree.iterator() iex&gt; {i1, iterator} = A.RBTree.next(iterator) iex&gt; {i2, iterator} = A.RBTree.next(iterator) iex&gt; A.RBTree.next(iterator) nil iex&gt; [i1, i2] [11, 22]","ref":"A.RBTree.html#iterator/1","title":"A.RBTree.iterator/1","type":"function"},{"doc":"Finds the value corresponding to the given key if exists.Examplesiex&gt; tree = A.RBTree.map_new(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}) iex&gt; A.RBTree.map_fetch(tree, :b) {:ok, &quot;B&quot;} iex&gt; A.RBTree.map_fetch(tree, :d) :error","ref":"A.RBTree.html#map_fetch/2","title":"A.RBTree.map_fetch/2","type":"function"},{"doc":"Inserts the key-value pair in a map tree and returns the updated tree.Returns a {:new, new_tree} tuple when the key was newly created. Returns a {{:overwrite, previous_value}, new_tree} tuple when the key was already present with the value previous_value.Examplesiex&gt; tree = A.RBTree.map_new(%{1 =&gt; &quot;A&quot;, 3 =&gt; &quot;C&quot;}) iex&gt; A.RBTree.map_insert(tree, 2, &quot;B&quot;) {:new, {:B, {:B, :E, {1, &quot;A&quot;}, :E}, {2, &quot;B&quot;}, {:B, :E, {3, &quot;C&quot;}, :E}}} iex&gt; A.RBTree.map_insert(tree, 3, &quot;C!!!&quot;) {{:overwrite, &quot;C&quot;}, {:B, :E, {1, &quot;A&quot;}, {:R, :E, {3, &quot;C!!!&quot;}, :E}}}","ref":"A.RBTree.html#map_insert/3","title":"A.RBTree.map_insert/3","type":"function"},{"doc":"Adds many key-values to an existing map tree, and returns both the new tree and the number of new entries created.Returns a {inserted, new_tree} tuple when inserted is the number of newly created entries. Updating existing keys do not count. This is useful to keep track of size changes.Examplesiex&gt; tree = A.RBTree.map_new(%{1 =&gt; &quot;A&quot;, 2 =&gt; &quot;B&quot;}) iex&gt; A.RBTree.map_insert_many(tree, %{2 =&gt; &quot;B&quot;, 3 =&gt; &quot;C&quot;}) {1, {:B, {:B, :E, {1, &quot;A&quot;}, :E}, {2, &quot;B&quot;}, {:B, :E, {3, &quot;C&quot;}, :E}}}","ref":"A.RBTree.html#map_insert_many/2","title":"A.RBTree.map_insert_many/2","type":"function"},{"doc":"Initializes a map tree from an enumerable.Examplesiex&gt; A.RBTree.map_new(%{1 =&gt; &quot;A&quot;, 2 =&gt; &quot;B&quot;, 3 =&gt; &quot;C&quot;}) {:B, {:B, :E, {1, &quot;A&quot;}, :E}, {2, &quot;B&quot;}, {:B, :E, {3, &quot;C&quot;}, :E}}","ref":"A.RBTree.html#map_new/1","title":"A.RBTree.map_new/1","type":"function"},{"doc":"Finds and removes the value corresponding for the given key if exists in a map tree, and returns both that value and the new tree.Uses the deletion algorithm as described in Deletion: The curse of the red-black tree.Examplesiex&gt; tree = A.RBTree.map_new(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}) iex&gt; {:ok, &quot;B&quot;, _new_tree} = A.RBTree.map_pop(tree, :b) {:ok, &quot;B&quot;, {:B, :E, {:a, &quot;A&quot;}, {:R, :E, {:c, &quot;C&quot;}, :E}}} iex&gt; :error = A.RBTree.map_pop(tree, :d) :error","ref":"A.RBTree.html#map_pop/2","title":"A.RBTree.map_pop/2","type":"function"},{"doc":"Finds and removes the rightmost (largest) key in a map tree.Returns both the key-value pair and the new tree.Examplesiex&gt; tree = A.RBTree.map_new(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}) iex&gt; {:ok, {:c, &quot;C&quot;}, new_tree} = A.RBTree.map_pop_max(tree) iex&gt; new_tree {:B, :E, {:a, &quot;A&quot;}, {:R, :E, {:b, &quot;B&quot;}, :E}} iex&gt; :error = A.RBTree.map_pop_max(A.RBTree.empty()) :error","ref":"A.RBTree.html#map_pop_max/1","title":"A.RBTree.map_pop_max/1","type":"function"},{"doc":"Finds and removes the leftmost (smallest) key in a map tree.Returns both the key-value pair and the new tree.Examplesiex&gt; tree = A.RBTree.map_new(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;}) iex&gt; {:ok, {:a, &quot;A&quot;}, new_tree} = A.RBTree.map_pop_min(tree) iex&gt; new_tree {:B, {:R, :E, {:b, &quot;B&quot;}, :E}, {:c, &quot;C&quot;}, :E} iex&gt; :error = A.RBTree.map_pop_min(A.RBTree.empty()) :error","ref":"A.RBTree.html#map_pop_min/1","title":"A.RBTree.map_pop_min/1","type":"function"},{"doc":"Finds the leftmost (smallest) element of a treeExamplesiex&gt; A.RBTree.map_new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.RBTree.max() {:ok, {:d, &quot;D&quot;}} iex&gt; A.RBTree.map_new([]) |&gt; A.RBTree.max() :error","ref":"A.RBTree.html#max/1","title":"A.RBTree.max/1","type":"function"},{"doc":"Finds the rightmost (largest) element of a treeExamplesiex&gt; A.RBTree.map_new([b: &quot;B&quot;, d: &quot;D&quot;, a: &quot;A&quot;, c: &quot;C&quot;]) |&gt; A.RBTree.min() {:ok, {:a, &quot;A&quot;}} iex&gt; A.RBTree.map_new([]) |&gt; A.RBTree.min() :error","ref":"A.RBTree.html#min/1","title":"A.RBTree.min/1","type":"function"},{"doc":"Walk a tree using an iterator yielded by iterator/1.Examplesiex&gt; iterator = A.RBTree.set_new([22, 11]) |&gt; A.RBTree.iterator() iex&gt; {i1, iterator} = A.RBTree.next(iterator) iex&gt; {i2, iterator} = A.RBTree.next(iterator) iex&gt; A.RBTree.next(iterator) nil iex&gt; [i1, i2] [11, 22]","ref":"A.RBTree.html#next/1","title":"A.RBTree.next/1","type":"function"},{"doc":"Computes the &quot;length&quot; of the tree by looping and counting each node.Examplesiex&gt; tree = A.RBTree.set_new([1, 2, 2.0, 3, 3.0, 3]) iex&gt; A.RBTree.node_count(tree) 3 iex&gt; A.RBTree.node_count(A.RBTree.empty()) 0","ref":"A.RBTree.html#node_count/1","title":"A.RBTree.node_count/1","type":"function"},{"doc":"Helper to implement Enumerable.reduce/3 in data structures using the underlying tree.","ref":"A.RBTree.html#reduce/3","title":"A.RBTree.reduce/3","type":"function"},{"doc":"Finds and removes the given value if exists, and returns the new tree.Uses the deletion algorithm as described in Deletion: The curse of the red-black tree.Examplesiex&gt; tree = A.RBTree.set_new([1, 2, 3, 4]) iex&gt; {:ok, _new_tree} = A.RBTree.set_delete(tree, 3) {:ok, {:B, {:B, :E, 1, :E}, 2, {:B, :E, 4, :E}}} iex&gt; :error = A.RBTree.set_delete(tree, 0) :error","ref":"A.RBTree.html#set_delete/2","title":"A.RBTree.set_delete/2","type":"function"},{"doc":"Inserts the value in a set tree and returns the updated tree.Returns a {:new, new_tree} tuple when the value was newly inserted. Returns a {:overwrite, new_tree} tuple when a non-striclty equal value was already present.Because 1.0 and 1 compare as equal values, inserting 1.0 can overwrite 1 and new_tree is going to be different.Examplesiex&gt; tree = A.RBTree.set_new([1, 3]) iex&gt; A.RBTree.set_insert(tree, 2) {:new, {:B, {:B, :E, 1, :E}, 2, {:B, :E, 3, :E}}} iex&gt; A.RBTree.set_insert(tree, 3.0) {:overwrite, {:B, :E, 1, {:R, :E, 3.0, :E}}}","ref":"A.RBTree.html#set_insert/2","title":"A.RBTree.set_insert/2","type":"function"},{"doc":"Adds many values to an existing set tree, and returns both the new tree and the number of newly inserted values.Returns a {inserted, new_tree} tuple when inserted is the number of newly inserted values. Overwriting existing values do not count. This is useful to keep track of size changes.Examplesiex&gt; tree = A.RBTree.set_new([1, 2]) iex&gt; A.RBTree.set_insert_many(tree, [2, 2.0, 3, 3.0]) {1, {:B, {:B, :E, 1, :E}, 2.0, {:B, :E, 3.0, :E}}}","ref":"A.RBTree.html#set_insert_many/2","title":"A.RBTree.set_insert_many/2","type":"function"},{"doc":"Checks the presence of a value in a set.Like all A.RBTree functions, uses ==/2 for comparison, not strict equality ===/2.Examplesiex&gt; tree = A.RBTree.set_new([1, 2, 3]) iex&gt; A.RBTree.set_member?(tree, 2) true iex&gt; A.RBTree.set_member?(tree, 4) false iex&gt; A.RBTree.set_member?(tree, 2.0) true","ref":"A.RBTree.html#set_member?/2","title":"A.RBTree.set_member?/2","type":"function"},{"doc":"Initializes a set tree from an enumerable.Examplesiex&gt; A.RBTree.set_new([3, 2, 1, 2, 3]) {:B, {:B, :E, 1, :E}, 2, {:B, :E, 3, :E}}","ref":"A.RBTree.html#set_new/1","title":"A.RBTree.set_new/1","type":"function"},{"doc":"Finds and removes the rightmost (largest) element in a set tree.Returns both the element and the new tree.Examplesiex&gt; tree = A.RBTree.set_new([1, 2, 3, 4]) iex&gt; {:ok, 4, new_tree} = A.RBTree.set_pop_max(tree) iex&gt; new_tree {:B, {:B, :E, 1, :E}, 2, {:B, :E, 3, :E}} iex&gt; :error = A.RBTree.set_pop_max(A.RBTree.empty()) :error","ref":"A.RBTree.html#set_pop_max/1","title":"A.RBTree.set_pop_max/1","type":"function"},{"doc":"Finds and removes the leftmost (smallest) element in a set tree.Returns both the element and the new tree.Examplesiex&gt; tree = A.RBTree.set_new([1, 2, 3, 4]) iex&gt; {:ok, 1, new_tree} = A.RBTree.set_pop_min(tree) iex&gt; new_tree {:B, {:R, :E, 2, :E}, 3, {:R, :E, 4, :E}} iex&gt; :error = A.RBTree.set_pop_min(A.RBTree.empty()) :error","ref":"A.RBTree.html#set_pop_min/1","title":"A.RBTree.set_pop_min/1","type":"function"},{"doc":"Returns the tree as a list.Examplesiex&gt; A.RBTree.set_new([3, 2, 2.0, 3, 3.0, 1, 3]) |&gt; A.RBTree.to_list() [1, 2.0, 3] iex&gt; A.RBTree.map_new([b: &quot;B&quot;, c: &quot;C&quot;, a: &quot;A&quot;]) |&gt; A.RBTree.to_list() [{:a, &quot;A&quot;}, {:b, &quot;B&quot;}, {:c, &quot;C&quot;}] iex&gt; A.RBTree.empty() |&gt; A.RBTree.to_list() []","ref":"A.RBTree.html#to_list/1","title":"A.RBTree.to_list/1","type":"function"},{"doc":"","ref":"A.RBTree.html#t:color/0","title":"A.RBTree.color/0","type":"type"},{"doc":"","ref":"A.RBTree.html#t:elem/0","title":"A.RBTree.elem/0","type":"type"},{"doc":"","ref":"A.RBTree.html#t:iterator/1","title":"A.RBTree.iterator/1","type":"type"},{"doc":"","ref":"A.RBTree.html#t:key/0","title":"A.RBTree.key/0","type":"type"},{"doc":"","ref":"A.RBTree.html#t:tree/0","title":"A.RBTree.tree/0","type":"type"},{"doc":"","ref":"A.RBTree.html#t:tree/1","title":"A.RBTree.tree/1","type":"type"},{"doc":"","ref":"A.RBTree.html#t:value/0","title":"A.RBTree.value/0","type":"type"},{"doc":"Some extra helper functions for working with strings, that are not in the core String module.","ref":"A.String.html","title":"A.String","type":"module"},{"doc":"Transforms the text as a slug. Removes whitespace, special characters and converts the rest to lowercase.This is typically useful to generate URLs based on content, e.g. the title of an article.Like String.downcase/2, slugify/2 also can handle the following modes: :default (keeps unicode), :ascii or :greek.Examplesiex&gt; A.String.slugify(&quot;&gt; \\&quot;It Was Me, Dio!!!\\&quot;\\n&quot;) &quot;it-was-me-dio&quot; iex&gt; A.String.slugify(&quot;Joseph Joestar a.k.a ジョジョ&quot;) &quot;joseph-joestar-aka-ジョジョ&quot; iex&gt; A.String.slugify(&lt;&lt;220&gt;&gt;) ** (ArgumentError) Invalid string &lt;&lt;220&gt;&gt;:ascii converts to ascii when possible or strips characters:iex&gt; A.String.slugify(&quot;OLÁ!\\n&quot;, :ascii) &quot;ola&quot; iex&gt; A.String.slugify(&quot;DIOの世界 -さらば友よ- &quot;, :ascii) &quot;dio&quot;:greek handles the context sensitive sigma in Greek:iex&gt; A.String.slugify(&quot;\\tΣΣ?&quot;) &quot;σσ&quot; iex&gt; A.String.slugify(&quot;\\tΣΣ?&quot;, :greek) &quot;σς&quot;","ref":"A.String.html#slugify/2","title":"A.String.slugify/2","type":"function"},{"doc":"","ref":"A.String.html#t:mode/0","title":"A.String.mode/0","type":"type"},{"doc":"Extension of the Elixir standard library focused on data stuctures and data manipulation.WARNING: Aja is still a work in progress. APIs might still change at any point.","ref":"readme.html","title":"README","type":"extras"},{"doc":"The Elixir standard library is so rich, ergonomic and well-documented that it feels there is not much left to add. And yet... some occasional friction might still remain.Aja aims to remove some of this friction by providing mostly:non-existing (e.g. ordered maps) or currently hard-to-use (e.g. binary search trees) data structures written in pure elixirnice-to-have utility functionsData structuresFriction can occur when the provided data structures do not satisfy the needs of a specific algorithm, be it for functionality or performance reasons. Sometimes, a linked-list just won't cut it. In the words of Okasaki:&quot;there is one aspect of functional programming that no amount of cleverness on the part of the compiler writer is likely to mitigate — the use of inferior or inappropriate data structures.&quot;Order maps (A.OrdMaps) are probably Aja's killer feature, since:regular maps do not keep track of the insertion orderkeywords do not have the right performance characteristics and only support atomsiex&gt; Map.new([{&quot;one&quot;, 1}, {&quot;two&quot;, 2}, {&quot;three&quot;, 3}]) |&gt; Enum.to_list() [{&quot;one&quot;, 1}, {&quot;three&quot;, 3}, {&quot;two&quot;, 2}] iex&gt; ord_map = A.OrdMap.new([{&quot;one&quot;, 1}, {&quot;two&quot;, 2}, {&quot;three&quot;, 3}]) #A&lt;ord(%{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3})&gt; iex&gt; ord_map[&quot;two&quot;] 2 iex&gt; Enum.to_list(ord_map) [{&quot;one&quot;, 1}, {&quot;two&quot;, 2}, {&quot;three&quot;, 3}]Order maps behave pretty much like regular maps, and the A.OrdMap module offers the same API as Map. They come with a convenience macro A.ord/1 to construct and pattern-match upon, which should make them a breeze to use without much added friction over plain maps:iex&gt; A.OrdMap.new(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3}) # without macro: insertion order is lost! #A&lt;ord(%{&quot;一&quot; =&gt; 1, &quot;三&quot; =&gt; 3, &quot;二&quot; =&gt; 2})&gt; iex&gt; import A iex&gt; ord_map = ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3}) # insertion order is preserved! #A&lt;ord(%{&quot;一&quot; =&gt; 1, &quot;二&quot; =&gt; 2, &quot;三&quot; =&gt; 3})&gt; iex&gt; ord(%{&quot;三&quot; =&gt; three, &quot;一&quot; =&gt; one}) = ord_map iex&gt; {one, three} {1, 3}Maps and sets based on Red-Black Trees (A.RBMap and A.RBSet) are useful when you want to keep a collection sorted.iex&gt; A.RBMap.new([b: &quot;Bat&quot;, a: &quot;Ant&quot;, c: &quot;Cat&quot;, b: &quot;Buffalo&quot;]) #A.RBMap&lt;%{a: &quot;Ant&quot;, b: &quot;Buffalo&quot;, c: &quot;Cat&quot;}&gt; iex&gt; A.RBSet.new([6, 6, 7, 7, 4, 1, 2, 3, 1, 5]) #A.RBSet&lt;[1, 2, 3, 4, 5, 6, 7]&gt;They offer similar functionalities as general balanced trees (:gb_trees and :gb_sets) included in the Erlang standard library. A.RBMap and A.RBSet should however offer better performance, and be more convenient to use.All those data structures offer:good performance characteristics at any size (see FAQ)well-documented APIs that are consistent with the standard libraryimplementation of Inspect, Enumerable and Collectable protocols(except for sets) implementation of the Access behaviour(optional if Jason is installed) implemention the Jason.Encoder protocolUtility functionsDon't Break The Pipe!iex&gt; %{foo: &quot;bar&quot;} |&gt; A.Pair.wrap(:noreply) {:noreply, %{foo: &quot;bar&quot;}} iex&gt; {:ok, 55} |&gt; A.Pair.unwrap!(:ok) 55Exclusive ranges (A.ExRange)iex&gt; A.ExRange.new(0, 10) |&gt; Enum.to_list() [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] iex&gt; import A iex&gt; Enum.map(0 ~&gt; 5, &amp;&quot;id_\#{&amp;1}&quot;) [&quot;id_0&quot;, &quot;id_1&quot;, &quot;id_2&quot;, &quot;id_3&quot;, &quot;id_4&quot;]Other helper examples:iex&gt; A.String.slugify(&quot;&gt; \\&quot;It Was Me, Dio!!!\\&quot;\\n&quot;) &quot;it-was-me-dio&quot; iex&gt; A.Integer.decimal_format(1234567) &quot;1,234,567&quot; iex&gt; A.Integer.div_rem(7, 3) {2, 1} iex&gt; A.List.repeatedly(&amp;:rand.uniform/0, 3) [0.40502929729990744, 0.45336720247823126, 0.04094511692041057] iex&gt; A.IO.iodata_empty?([&quot;&quot;, []]) trueNone of this is revolutionary, but having these helpers to hand might save you the implementation and the testing, or bringing over a library just for this one thing.All those helpers should provide some rationale in their documentation as to why they might be useful and what pain point they are addressing.Browse the API documentation to see if you find something that helps you.","ref":"readme.html#tl-dr","title":"README - TL;DR","type":"extras"},{"doc":"Aja can be installed by adding aja to your list of dependencies in mix.exs:def deps do [ {:aja, &quot;~&gt; 0.2.0&quot;} ] endDocumentation can be found at https://hexdocs.pm/aja.","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Inspirationsthe amazingly polished Elixir standard library: self-consistent, well-documented and just delightful ✨️the also amazing Python standard library, notably its collection modulethe amazing lodash which complements nicely the (historically rather small) javascript standard library, with a very consistent APIvarious work on efficient persistent data structures spearheaded by Okasaki (see resources section below)Goalslike the standard library, being delightful to use ✨️ (consistency, quality, documentation)no external dependency to help you preserve a decent dependency tree and fast compile timesperformance-conscious (right algorithm, proper benchmarking, fast compile times)mostly dead-simple pure functions: no configuration, no mandatory macro, no statefulness / OTPDon't Break The Pipe!: APIs have been designed with pipe-ability in mindNon-goalsadd every possible feature that has not been accepted in elixir core (Aja is opinionated!)touching anything OTP-related / statefulResourcesChris Okasaki's Purely Functional Data StructuresDeletion: The curse of the red-black tree by German and Might.","ref":"readme.html#about-aja","title":"README - About Aja","type":"extras"},{"doc":"How is the performance?Performance for maps is still far from native maps (roughly 4 times slower insertions) or ETS (mutable state).Aja data structures are implemented in plain erlang/elixir and cannot compete with native code yet.However:A.RBMap / A.OrdMap / A.RBSet perform better than the also non-native :gb_trees / :gb_sets modules (as promised by Okasaki, &quot;it really flies!&quot;)the performance gap is consistent and doesn't degrade with the size (logarithmic time complexity)with the JIT compilation coming to the BEAM, we can expected the gap with native code to be reduced in the upcoming months.Aja data structures should work fine in most cases, but if you're considering them for performance-critical sections of your code, make sure to benchmark them and also consider alternatives, typically ETS if mutable state is acceptable.Benchmarking is still a work in progress, but you can check the bench folder for more detailed figures.Why is there a convenience macro for A.OrdMap but not for other structures?There are actually two reasons for this:ordered maps would be unconvenient to initialize otherwiseordered maps can be pattern-matched upon due to their internal representation, tree-based structures cannot1. Initialization with new/1:Ordered maps are tricky to initialize, and A.OrdMap.new/1 is not convenient to do so. We cannot simply pass it a map, because the map will reorder the keys. We have to pass it a list of tuples, which is fine if keys are atoms, but feels messy and not readable otherwise.Being a macro, A.ord/1 is able to read the code and preserve the order, without ever instanciating a map that would lose the order:iex&gt; A.OrdMap.new(%{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3}) #A&lt;ord(%{&quot;one&quot; =&gt; 1, &quot;three&quot; =&gt; 3, &quot;two&quot; =&gt; 2})&gt; iex&gt; ord(%{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3}) #A&lt;ord(%{&quot;one&quot; =&gt; 1, &quot;two&quot; =&gt; 2, &quot;three&quot; =&gt; 3})&gt;A.RBMap.new/1, A.RBSet.new/1 ... do not face any similar constraints and wouldn't benefit from a macro.2. Pattern-matchingShort answer: because the internal representation of ordered maps happens to use a map, it is possible to make A.ord/1 work as it does. Tree-based A.RBMaps cannot enjoy this treatment.Longer answer: Elixir (Erlang) is limited in what can be pattern-matched upon, because it does not offer active patterns. While this is a fine decision that helps keeping the language simpler, it has the drawback of being tied to the internal representation of data structures.Quoting Okasaki again, describing what might be called pattern-matching induced damage:&quot;Ironically, pattern matching — one of the most popular features in functional programming languages — is also one of the biggest obstacles to the widespread use of efficient functional data structures. The problem is that pattern matching can only be performed on data structures whose representation is known, yet the basic software-engineering principle of abstraction tells us that the representation of non-trivial data structures should be hidden. The seductive allure of pattern matching leads many functional programmers to abandon sophisticated data structures in favor of simple, known representations such as lists, even when doing so causes an otherwise linear algorithm to explode to quadratic or even exponential time.&quot;Making pattern-matching work for trees would probably need to implement some kind of active pattern, that would imply to redefine alternative versions of def, case and =/2.Does Aja try to do too much?The Unix philosophy of &quot;Do one thing and do it well&quot; is arguably the right approach in many cases. Aja doesn't really follow it, but there are conscious reasons for going that direction.While it might be possible later down the road to split some of its components, there is no plan to do so at the moment.First, we don't think there is any real downside of shipping &quot;too much&quot;: Aja has and aims to keep a lightweight footprint and fast compile times, as well as a modular structure. You can just use what you need without suffering from what you don't.This lodash-like approach has benefits too: it aims to ship with a lot of convenience while introducing only one flat dependency. This can help staying out of two extreme paths:the &quot;leftpad way&quot;, where every project relies on a ton of small dependencies, ending up with un-manageable dependency trees and brittle software.the &quot;Lisp Curse way&quot;, where everybody keeps rewriting the same thing over and over because nobody wants the extra dependency. Being a hidden Lisp with similar super powers and expressiveness, Elixir might make it relatively easy and tempting to go down that path.What are the next steps?Nothing is set in stone, but the next steps will probably be:keep working towards production-readiness: testing, improve documentationmore benchmarks and performance optimizationsadd an OrderedSetevaluate Kahrs algorithm as an alternative for red-black tree deletionevaluate some other interesting data structures to add (clojure's vectors or RRB Trees)","ref":"readme.html#faq","title":"README - FAQ","type":"extras"},{"doc":"Aja is licensed under the MIT License.","ref":"readme.html#copyright-and-license","title":"README - Copyright and License","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"EnhancementsAdd A.String.slugify/2Breaking changesRemove A.Array module","ref":"changelog.html#v0-2-0-2020-10-25","title":"Changelog - v0.2.0 (2020-10-25)","type":"extras"},{"doc":"EnhancementsAdd pop_first/1 and pop_last/1 to A.RBMap, A.RBSet and A.OrdMapAdd some guards to functionsBug fixesA.RBSet.disjoint?/2 was not returning the expected value","ref":"changelog.html#v0-1-2-2020-10-22","title":"Changelog - v0.1.2 (2020-10-22)","type":"extras"},{"doc":"Bug fixesFix incompatibility with Elixir 1.10","ref":"changelog.html#v0-1-1-2020-10-21","title":"Changelog - v0.1.1 (2020-10-21)","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0-2020-10-18","title":"Changelog - v0.1.0 (2020-10-18)","type":"extras"},{"doc":"MIT LicenseCopyright (c) 2020 SabiwaraPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]